<?php

namespace Drupal\marketing_cloud_example\Plugin\WebformHandler;

use Drupal\webform\WebformSubmissionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\webform\Plugin\WebformHandlerBase;

/**
 * Form submission handler.
 *
 * @WebformHandler(
 *   id = "marketing_cloud_example",
 *   label = @Translation("Marketing Cloud Example"),
 *   category = @Translation("Form Handler"),
 *   description = @Translation("Send an SMS on form submission"),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_SINGLE,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 *   submission = \Drupal\webform\Plugin\WebformHandlerInterface::SUBMISSION_REQUIRED,
 * )
 */
class MarketingCloudExample extends WebformHandlerBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
      'short_code' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['short_code'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Short Code'),
      '#description' => $this->t('Short code  - this is the base message code in SF that we will override.'),
      '#default_value' => $this->configuration['short_code'],
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    if (!$form_state->getErrors()) {
      $this->configuration['short_code'] = $form_state->getValue('short_code');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state, WebformSubmissionInterface $webform_submission) {
    $values = $webform_submission->getData();
    $mobileNumbers = !is_array($values['mobile_number']) ? [$values['mobile_number']] : $values['mobile_number'];
    $date = new \DateTime();
    // Arbitrary time in the past to send instantly.
    $date->modify('-1 day');
    $messageId = $this->configuration['short_code'];
    $data = [
      'mobileNumbers' => $mobileNumbers,
      'override' => TRUE,
      'messageText' => $values['message_text'],
      'sendTime' => $date->format('Y-m-d H:i'),
    ];
    $response = \Drupal::service('marketing_cloud_sms.service')
      ->postMessageToNumber($messageId, $data);
    if (!$response) {
      $message = t('SMS send failed. Please check the log.');
      drupal_set_message($message, 'info');
    }
    elseif (isset($response['errors'])) {
      $message = t('Message not sent - errors: %errors', ['%errors' => $response['errors']]);
      drupal_set_message($message, 'error');
      \Drupal::logger(__METHOD__)->error($message);
    }
    elseif (!isset($response['result'])) {
      $message = t('Message not sent - errors: %errors', ['%errors' => print_r($response, TRUE)]);
      drupal_set_message($message, 'error');
      \Drupal::logger(__METHOD__)->error($message);
    }
    else {
      $message = t('Response from SalesForce: "%result"', ['%result' => $response['result']]);
      drupal_set_message($message, 'info');
      \Drupal::logger(__METHOD__)->info($message);
    }
  }

}
